require 'rpmmodule'
require 'rbconfig'
require 'ftools'
require 'singleton'

sUname = Config::CONFIG['arch'].split(/\-/).[](0)
if sUname =~ /i.86/ then
  ARCH = 'i686'
elsif sUname =~ /powerpc/ then
  ARCH = 'ppc'
else
  ARCH = sUname
end

module RPM
  class Dependency
    def to_s(type = "nazo no dependency")
      str = "#{owner} #{type} #{name}"
      if flags != 0
        if ( lt? && !gt? && !eq? && !le? && !ge? )    # <
          str << " < #{version}"
        elsif ( !lt? && gt? && !eq? && !le? && !ge? ) # >
          str << " > #{version}"
        elsif ( !lt? && !gt? && eq? && !le? && !ge? ) # =
          str << " = #{version}"
        elsif ( lt? && !gt? && eq? && le? && !ge? )   # <=
          str << " <= #{version}"
        elsif ( !lt? && gt? && eq? && !le? && ge? )   # >=
          str << " >= #{version}"
        end
      end
      str
    end
  end

  class Require
    def to_s
      super("requires")
    end
  end

  class Conflict
    def to_s
      super("conflicts")
    end
  end

  class Obsolete
    def to_s
      super("obsoletes")
    end
  end

  class Provide
    def to_s
      super("provides")
    end
  end

  class Package
    attr_writer :location
    attr_accessor :size
    attr_writer :filename

    def <=>(other)
      if self == other
        ret = 0
      else
        ret = (name <=> other.name)
        if ret == 0
          ret = (version <=> other.version)
        end
      end
      ret
    end

    def ==(other)
      if (other.is_a? RPM::Package)
        self.sigmd5 == other.sigmd5
      elsif other.is_a? String
        name == other
      else
        false
      end
    end

    def is_kernel?
      if (name  =~ /^kernel/ &&
          name !~ /^kernel(24|26)?-(common|pcmcia-cs|doc|utils)/ )
        true
      else
        false
      end
    end

    def sigmd5
      self[RPM::TAG_SIGMD5]
    end

    def filename
      if(@location && @location[0..0] == '/')
        return @location + "/" + @filename
      end
      @filename
    end
  end

  class DB
    def DB.new_with_wait(writable = false)
      db = RPM::DB.new(writable)
      if db == nil
        if Thread.list.length >1
          Thread.pass
        elese
          MPH::Msg.instance.print(MPH::MsgLevel::NORMAL,"waiting for rpm db..\n")
          sleep 1
        end
        retry 
      end
      db
    rescue RuntimeError
      if Thread.list.length >1
        Thread.pass
      elese
        MPH::Msg.instance.print(MPH::MsgLevel::NORMAL,"waiting for rpm db..\n")
        sleep 1
      end
      retry 
    end
  end
end

module MPH
  begin
    require 'zlib'
    HaveZlib = true
  rescue LoadError
    HaveZlib = false
  end

  VERSION = "0.92.7"

  require 'mph/transfer'

  def set_get_commands(astr)
    @get_commands = []
    astr.each do |str|
      @get_commands << MPH::Transfer.new(str)
    end
  end
  private :set_get_commands
  module_function :set_get_commands

  def get_remote_file(sSrc)
    @get_commands.each do |conection|
      return true if conection.get_file(sSrc)
    end
  end
  private :get_remote_file
  module_function :get_remote_file

  def close_connection(sSrc)
    @get_commands.each do |conection|
      conection.close
    end
  end
  private :get_remote_file
  module_function :get_remote_file

  module MsgLevel
    ERROR=0
    NORMAL=1
    VERBOSE=2
    LOG=3
    DEBUG=4
  end

  class Msg
    include Singleton

    attr_accessor :level
    attr_accessor :loglevel

    def print(level,arg)
      if level <= @level
        super(arg)
      end
      if @log && (level <= @loglevel)
        @log.print(arg)
        @log.flush
      end
    end

    def clean
      @log.close
      File.unlink(@logfile)
    end

    def init_log(logfile = "/var/cache/mph/resolve.log")
      @logfile = logfile
      begin
        @log = open(@logfile,"w")
      rescue
      end
    end

    def confirm(msg,force=false)
      if !force
        print(MsgLevel::NORMAL, "#{msg} (y/n)> " )
	$defout.flush
        sAnswer = STDIN.gets().chomp().downcase()
        if( sAnswer != 'y' )
          return false
        end
      end
      true
    end

    def initialize
      @level = MsgLevel::NORMAL
      @loglevel = MsgLevel::LOG
      @log = nil
      @logfile = nil
    end
  end

  class DB<Object
    include Enumerable

    def each
      pr = Proc.new
      @mphPackages.each do |k,v|
        v.each(&pr)
      end
    end

    def each_match(tag,val)
      pr = Proc.new
      packages = nil
      case tag
      when RPM::DBI_LABEL
        packages = @mphPackages[val]
      when RPM::TAG_REQUIRENAME
        packages = @requires_cache[val]
      when RPM::TAG_PROVIDENAME
        packages = @provides_cache[val]
      when RPM::TAG_OBSOLETENAME
        packages = @obsoletes_cache[val]
      when RPM::TAG_CONFLICTNAME
        packages = @conflicts_cache[val]
      else
        packages = find_all{|p| p[tag] == val}
      end
      if packages
        packages.each(&pr)
      end
    end

    def push(package,rpmdb,dont_use_hold = false, with_kernel = false)
      if(!dont_use_hold)
        if @holds.index(package.name)
          @msg.print(MsgLevel::VERBOSE, "found Holds package #{package}.\n")
          return
        end
      end
      if package.is_kernel?
        @msg.print(MsgLevel::VERBOSE, "found kernel package #{package}.\n")
        if !with_kernel
          return
        end
        mi = rpmdb.init_iterator(RPM::DBI_LABEL,package.name)
	if mi then
          mi.set_iterator_version(package.version)
          localkp = mi.next_iterator
          if localkp and localkp.version == package.version then
            return
          end
        end
      end
      (@mphPackages[package.name] ||= []) << package
      package.provides.each do |dep|
        (@provides_cache[dep.name] ||= []) << package
      end
      package.requires.each do |dep|
        (@requires_cache[dep.name] ||= []) << package
      end
      package.conflicts.each do |dep|
        (@conflicts_cache[dep.name] ||= []) << package
      end
      package.obsoletes.each do |dep|
        (@obsoletes_cache[dep.name] ||= []) << package
      end
    end
    private :push

    def setup_db(loaders = [DBLoader],dont_get_dbfile = false,dont_use_hold = false,with_kernel = false, search_noexist_mode = false)
      rpmdb = RPM::DB.new_with_wait
      @directories.each_with_index do |d,i|
        loader = nil
        loaders.each do |dbloader|
          begin
            loader = dbloader.new(d,"#{@sDBFile}#{i}",dont_get_dbfile)
            break
          rescue
            loader = nil
          end
        end
        return false if !loader
        loader.each do |p|
          if ( search_noexist_mode || ((p.sigmd5) && (!rpmdb.init_iterator(RPM::TAG_SIGMD5,p.sigmd5))))
            push(p,rpmdb,dont_use_hold, with_kernel)
          end
        end
      end
      rpmdb = nil
      GC.start
      true
    end

    def clean
      `rm -rf #{@sCacheDir}`
      `rm -rf #{@sLibDir}tmp`
      @msg.clean
      File.mkpath @sCacheDir
    end

    def check_old_mph_conf
      #//------------------------------------------------
      # Sanity checks
      bSuccess = true
      [
        [ '$get_commands', [Array], 'an array of strings' ],
        [ '$keep_local_copy', [TrueClass, FalseClass], 'a truth value' ],
        [ '$directories', [Array], 'an array of strings' ],
        [ '$holds', [Array], 'an array of strings' ],
      ].each do |sName, tType, sType|
        oVar = eval( sName )
        if( tType.index( oVar.class ) == nil )
          @msg.print(MsgLevel::ERROR,"#{sConfFile}: #{sName} must be #{sType}.\n")
          bSuccess = false
        elsif( oVar.class == Array )
          oVar.each do |sItem|
            if( sItem.class != String )
              @msg.print(MsgLevel::ERROR,"#{sConfFile}: #{sName} must be #{sType}.\n")
              bSuccess = false
              break
            end
          end
        end
      end
      if( ! bSuccess )
        exit( -1 )
      end
      MPH::set_get_commands( $get_commands)
      @keep_local_copy = $keep_local_copy
      @directories = $directories
      @holds = $holds
    end

    def initialize(sConfFile='/etc/mph.conf', force_mode = false)
      # Hash Ȥ RPM::Package  Array

      @msg = Msg.instance
      #toriaezu
      @msg.init_log("/var/cache/mph/resolve.log")
      
      @mphPackages = {}
      @provides_cache = {}
      @conflicts_cache = {}
      @requires_cache = {}
      @obsoletes_cache = {}

      @force_mode = force_mode

      $get_commands = nil
      $keep_local_copy = nil
      $directories = nil
      begin
        load( sConfFile )
      rescue NameError
#        check_new_mph_conf
         exit -1
      rescue LoadError
        @msg.print(MsgLevel::ERROR,"#{sConfFile} load error\n")
        exit(-1)
      else
        check_old_mph_conf
      end

      @sMphDir    = '/var/cache/mph/'
      @sTmpDir    = '/tmp/'
      @sLibDir   = @sMphDir + 'lib/'
      @sCacheDir = @sMphDir + 'cache/'
      @sTmpFile  = @sMphDir + 'tmp'
      @sDBFile   = @sLibDir + 'Packages'
      Dir.chdir(@sLibDir)
    end
  end

  class DBLoader
    def get_dbfile(sDir,localpath)
      if sDir[0..0]!='/'
        @msg.print(MsgLevel::NORMAL,"Updating databases #{sDir}\n" )
        f = File.open(localpath+"new","w")
        sRemote = sDir + @sufix
        sLocal = sRemote.sub( %r|.*/|, '' )
        if( ! MPH::get_remote_file( sRemote ) )
          @msg.print( MsgLevel::ERROR,"Failed to get #{sRemote}.\n" )
          return false
        end
        f.print("LOCATION #{sDir}\n\n")
        rf = nil
        begin
          rf = @io.open(sLocal)
          f.write(rf.read)
        rescue Zlib::Error
            return false
        end
        rf.close
        File.unlink(sLocal)
        f.close
        File.rename(localpath+"new",localpath)
      end
      true
    end
    private :get_dbfile

    def parse_dbfile(io,location)
      begin
        while true
          filename = io.readline.strip
          size = Integer(io.readline.strip)
          package = Marshal.load(io)
          package.location = location
          package.size = size
          package.filename = filename
          @packages << package
        end
      rescue EOFError
      rescue
        return false
      end
      true
    end
    private :parse_dbfile

    def each(&b)
      @packages.each(&b)
    end

    def initialize(dbpath,localpath,dont_get_dbfile = false,sufix = ".mphrpm",io = Zlib::GzipReader)
      @msg = Msg.instance
      @packages = []
      @sufix = sufix
      @io = io
      if !dont_get_dbfile
        File.rm_f("#{localpath}new") if File.exist?("#{localpath}new")
        if dbpath[0..0]=='/'
          require 'mph/scanpackages.rb'
          Scanpackages.scan(dbpath)
          File.open("#{localpath}new","w") do |o|
            @io.open("#{dbpath}#{sufix}") do |i|
              o.print("LOCATION #{dbpath}\n\n")
              o.write(i.read)
            end
          end
          File.rename(localpath+"new",localpath)
        else
          if !get_dbfile(dbpath,localpath)
            raise "db loader"
          end
        end
      end
      @msg.print(MsgLevel::NORMAL, "Reading database #{dbpath}\n" )
      ioFile = open( localpath, 'r' )
      @msg.print(MsgLevel::VERBOSE, "reading database: #{localpath}\n")

      location = ioFile.readline.chomp().split( / / )[1]
      ioFile.readline
      if !parse_dbfile(ioFile,location)
        raise "db loader"
      end
    end
  end
end

### :nodoc:
### Local Variables:
### mode: ruby
### ruby-indent-level: 2
### indent-tabs-mode: nil
### End:
